#!/usr/bin/muawk
# mode.awk (rustic) Xfree modeline generator
# (C) 2001, m. Andreoli for MuLinux

# Based on info extracted reading the
# Xfree86-Video-Timing-HOWTO, by Eric S. Raymond


BEGIN	{
	n=split(arg,opts)	
	a1=1.25; a2=1.09;
	A=1.65; B=7.3e5;
	a=4.0/3.0; 	# aspect ratio

	# default
	verbose=0
	fh1=29; fh2=60
	fv1=47; fv2=100	
	H=800; V=600	# 640x480, 800x600, 1024x768, 1280x1024

	#------------
	# syntax
	#-------------

	if ( opts[1] == "-h" || n==0 ) {
		printf("XFREE86 Modeline Rustic Computing - M.Andreoli\n");
		printf("Usage1: mode [-v] HxV clock-range \n");
		printf("Usage2: mode [-v] -[all|best|std] clock-range \n");
		printf("Usage3: mode -default (default fixed modes)\n");
		printf("Example: mode 800x600 20-70 \n"); 
		printf("Example: mode -all 20-70\n"); 
		printf("Example: mode -std 65.2\n"); 
		printf("Example: mode -best 20-80\n"); 
		printf("Example: mode -u 58.9 800 808 912 1040   600 602 610 630\n"); 
		exit
	}
	k=1;

	if ( opts[k] == "-default" ) {
		system("cat /usr/X11R6/lib/X11/config/XF86Config.modes");
		exit(0);
	}

	if ( opts[k] == "-v" ) {
		k++;verbose=1
	}

	if ( opts[k] == "-best" ) {
		k++;
		get_clock(k);k++;
		best()
		exit(0)
	}


	if ( opts[k] == "-u" ) {
                k++; user_supplied(); exit(0)
        }



	if ( opts[k] == "-std" ) {
                k++;
		get_clock(k);k++;
		best_standard()
                exit(0)
        }


	# option -all ( print all modelines)

	if ( opts[k] == "-all" ) {
	k++;
	get_clock(k);k++;
	compute_all()
	exit(0)
	}


	# print a specific modeline

	if ( n!= 0 ) {
	get_res(k)
	k++;get_clock(k);
	specific()
	exit(0);
	}

	}

function specific()
{
for (f0=fh1; f0<=fh2; f0=f0+step(fh1,fh2) )
	modeline(H,V, name(H,V));
}




function user_supplied()
{
printf("Modeline \"%s\"","best")
printf(" %4.1f",opts[2]);

for (i=3; i<=10; i++)
	printf(" %d",opts[i])
printf("\n");
}

function step(x1,x2)
{
tot=40.0
if ( x2 > x1 )
	return (x2-x1)/tot
else
	return 1
}


function best()
{

for (f0=fh1; f0<=fh2; f0=f0+step(fh1,fh2) )
{
f=60; # vesa ergonomic
R=B*f0/f
H=int(sqrt(a*R))
# round to multiple of 32
r=H % 32; H = H - r
V=int(H/a)
modeline(H,V, "best")
}
}

function compute_all()
{
for (f0=fh1;f0<=fh2;f0=f0+step(fh1,fh2))
{
        modeline(640,480, name(640,480));
        modeline(800,600,name(800,600));
        modeline(1024,768,name(1024,768));
        modeline(1280,1024,name(1280,1024));
}

}

function best_standard()
{
for (f0=fh1;f0<=fh2;f0=f0+step(fh1,fh2))
{
f=75; # vesa ergonomic
R=B*f0/f
h=int(sqrt(a*R))

if ( h < 700 )  H=640
else
	if ( h < 900 ) H=800
		else
			if ( h < 1100 ) H=1024
				else H=1280 

# round to multiple of 32
r=H % 32; H = H - r
V=int(H/a)
modeline(H,V, name(H,V))
}
}



function name(h,v)
{
return h "x" v
}

function get_res(k)
{
split(opts[k],field,"x")
H=field[1];
V=field[2];
}


function get_clock(k)
{
split(opts[k],field,"-")
fh1=field[1];
if ( field[2] )
	 fh2=field[2]
else
	fh2=fh1

}

function modeline(h,v,N)
{
R=h*v;
vmin=fh1*B/R
vmax=fh2*B/R
if (verbose==1)
printf("# *** NOTE: These modes spans VertRefresh in %d-%d .\n",vmin,vmax)

f=f0*B/R
fh=f0/sqrt(A/B*R)

msg="ok"; status=0;
if ( f < 60.0 ) { msg="flickers!" }
if (  f < fv1 ) { msg="skip <"; status=1}
if (  f > fv2 ) { msg="skip >"; status=1}

if (verbose==1)
 printf("# (%dx%d) RefreshRate=%2.0f Hz, \
hsync=%4.1f kHz, clock=%4.1f Mhz - [%s]\n",h,v, f,fh,f0,msg);

	 printf("Modeline \"%s\" %4.1f %s\n",N, f0,\
	 timing(h,v,f0));
}

# il timing orizzontale dev'essere multiplo di 8
# norm "normalizza" al multiplo di 8 piu' piccolo

function norm(x)
{
r=x % 8;
return x-r;
}


function set_porch(clock) {
# all in micro-seconds 
if ( clock < 30 ) {
	k1=0.03; k2=0.12
	a1=1.25
	a2=1.09
} 
else {
	f=30; 
	k1=0.01; k2=0.10
	a1=1.3
	a2=1.05
}
		
}


function timing(h,v,f0)
{
set_porch(f0)
# horiz timings

h0=norm(h*a1)
n=h0/8
n1=int(k1*n); n2=int(k2*n)
h1=h+8*n1
h2=h1+8*n2
# vert timings
v0=int(a2*v);
tick=h0/f0      # line duration (in us)
v1= v + 2 
v2=int ( v1 + 150/tick )



b=" "
return h b  h1 b h2  b  h0 b b b        v  b v1 b v2 b v0;
}



function timing_old(h,v,f0)
{
# horiz timings

h0=norm(int(h*a1))
n=h0/8 
h1=h+int(n*0.01)*8
h2=h1+int(n*0.10)*8

# vert timings
tick=h0/f0	# durata di una riga (in us)
k=int(150/tick)	# ticks corrispondenti ad un impulso di 150 us

v0=int(a2*v);
v1=v+1;
v2=v1+k;
b=" "
return h b  h1 b h2  b  h0 b b b        v  b v1 b v2 b v0;
}

 
